#written by John Hoffman


false = 0
true = not false

from ConnChoice import *
from wxPython.wx import *
from types import *
from sys import *



class configReader:

    def __init__(self, defaults):
        self.defaults = defaults
        self.configfile = wxConfig("BitTorrent")
        self.configMenuBox = None
        self.configReset = true         # run reset for the first time

        self.configFileDefaults = {
            #args only available here:
            'win32_taskbar_icon': 1,
                # "whether to iconize do system try or not on win32"),
            'gui_stretchwindow': 0,
                # "whether to stretch the download status window to fit the torrent name"),
            'gui_displaystats': 1,
                # "whether to display statistics on peers and seeds"),
            'gui_displaymiscstats': 1,
                # "whether to display miscellaneous other statistics"),
            'gui_ratesettingsdefault': 'unlimited',
                # "the default setting for maximum upload rate and users"),
            'gui_ratesettingsmode': 'full',
                # "what rate setting controls to display; options are 'none', 'basic', and 'full'"),
            'gui_forcegreenonfirewall': 0,
                # "forces the status icon to be green even if the client seems to be firewalled")
            'gui_checkingcolor': 'FF FF FF',
            'gui_downloadcolor': '00 00 00',
            'gui_seedingcolor': 'FF 00 00'
            }

        self.configFileDefaults['gui_checkingcolor'] = self.ColorToHex(wx.wxSystemSettings_GetColour(wxSYS_COLOUR_3DSHADOW))
        self.configFileDefaults['gui_downloadcolor'] = self.ColorToHex(wx.wxSystemSettings_GetColour(wxSYS_COLOUR_ACTIVECAPTION))
        
        configfileargs = {
            # args listed in download.py:
            'minport': None,
            'maxport': None,
            'bind': None,
            }

        for name in self.configFileDefaults:
            configfileargs[name] = self.configFileDefaults[name]
        self.configfileargs = configfileargs


        for name in self.configfileargs:
            default = self.defaults.get(name)
            if default is None:
                default = self.configFileDefaults[name]
            if type(default) is IntType:
                if self.configfile.Exists(name):
                    setting = self.configfile.ReadInt(name,0)
                    if setting is None:
                        setting = default
                        self.configfile.WriteInt(name,default)
                    self.configfileargs[name] = setting
                else:
                    self.configfileargs[name] = default
                    self.configfile.WriteInt(name,default)
            elif type(default) is FloatType:
                if self.configfile.Exists(name):
                    setting = self.configfile.ReadFloat(name,0.0)
                    if setting is None:
                        setting = default
                        self.configfile.WriteFloat(name,default)
                    self.configfileargs[name] = setting
                else:
                    self.configfileargs[name] = default
                    self.configfile.WriteFloat(name,default)
            elif type(default) is StringType:
                if self.configfile.Exists(name):
                    setting = self.configfile.Read(name,'')
                    if (setting is None) or (setting == '') or (setting == 'None'):
                        setting = default
                        self.configfile.Write(name,default)
                    self.configfileargs[name] = setting
                else:
                    self.configfileargs[name] = default
                    self.configfile.Write(name,default)
            #else skip it...
        self.configfile.Flush()
                    
        self.checkingcolor = self.HexToColor(self.configfileargs['gui_checkingcolor'])
        self.downloadcolor = self.HexToColor(self.configfileargs['gui_downloadcolor'])
        self.seedingcolor = self.HexToColor(self.configfileargs['gui_seedingcolor'])
        

    def HexToColor(self, s):
        r,g,b = s.split(' ')
        return wxColour(red=int(r,16), green=int(g,16), blue=int(b,16))
        
        
    def ColorToHex(self, c):
        return hex(c.Red()) + ' ' + hex(c.Green()) + ' ' + hex(c.Blue())


    def resetConfigDefaults(self):
        for name in self.configFileDefaults:
            self.configfileargs[name] = self.configFileDefaults[name]
        for name in self.configfileargs:
            default = self.defaults.get(name)
            if default is not None:
                self.configfileargs[name] = default
        writeConfigFile()



    def writeConfigFile(self):
        for name in self.configfileargs:
            default = self.configfileargs[name]
            if type(default) is IntType:
                self.configfile.WriteInt(name,default)
            elif type(default) is FloatType:
                self.configfile.WriteFloat(name,default)
            else:  # assume StringType:
                self.configfile.Write(name,str(default))
        self.configfile.Flush()



    def setColorIcon(self, xxicon, xxiconptr, xxcolor):
        idata = wxMemoryDC()
        idata.SelectObject(xxicon)
        idata.SetBrush(wxBrush(xxcolor,wxSOLID))
        idata.DrawRectangle(0,0,16,16)
        idata.SelectObject(wxNullBitmap)
        xxiconptr.Refresh()


    def getColorFromUser(self, parent, colInit):
        data = wxColourData()
        if colInit.Ok():
            data.SetColour(colInit)
        data.SetCustomColour(0, self.checkingcolormenu)
        data.SetCustomColour(1, self.downloadcolormenu)
        data.SetCustomColour(2, self.seedingcolormenu)
        dlg = wxColourDialog(parent,data)
        if not dlg.ShowModal():
            return colInit
        return dlg.GetColourData().GetColour()
            



    def configMenu(self):
        self.checkingcolormenu = self.checkingcolor
        self.downloadcolormenu = self.downloadcolor
        self.seedingcolormenu = self.seedingcolor
        
        if (self.configMenuBox is not None):
            try:
                self.configMenuBox.Close ()
            except wxPyDeadObjectError, e:
                self.configMenuBox = None

        self.configMenuBox = wxFrame(None, -1, 'BitTorrent Preferences', size = (1,1))
#        if (sys.platform == 'win32'):
#            self.configMenuBox.SetIcon(self.icon)

        panel = wxPanel(self.configMenuBox, -1)
        self.panel = panel
        colsizer = wxFlexGridSizer(cols = 1, vgap = 13)

        gui_stretchwindow_checkbox = wxCheckBox(panel, -1, "Stretch window to fit torrent name *")
        gui_stretchwindow_checkbox.SetValue(self.configfileargs['gui_stretchwindow'])

        gui_displaystats_checkbox = wxCheckBox(panel, -1, "Display peer and seed statistics")
        gui_displaystats_checkbox.SetValue(self.configfileargs['gui_displaystats'])

        gui_displaymiscstats_checkbox = wxCheckBox(panel, -1, "Display miscellaneous other statistics")
        gui_displaymiscstats_checkbox.SetValue(self.configfileargs['gui_displaymiscstats'])
        
        gui_forcegreenonfirewall_checkbox = wxCheckBox(panel, -1,
                            "Force icon to display green when firewalled")
        gui_forcegreenonfirewall_checkbox.SetValue(self.configfileargs['gui_forcegreenonfirewall'])

        minport_data = wxSpinCtrl(panel, -1, '', (-1,-1), (60, -1))
        minport_data.SetRange(1,65535)
        minport_data.SetValue(self.configfileargs['minport'])

        maxport_data = wxSpinCtrl(panel, -1, '', (-1,-1), (60, -1))
        maxport_data.SetRange(1,65535)
        maxport_data.SetValue(self.configfileargs['maxport'])
        
        gui_ratesettingsdefault_data=wxChoice(panel, -1, choices = connChoiceList )
        gui_ratesettingsdefault_data.SetStringSelection(self.configfileargs['gui_ratesettingsdefault'])

        gui_ratesettingsmode_data=wxRadioBox(panel, -1, 'Rate Settings Mode',
                 choices = [ 'none', 'basic', 'full' ] )
        gui_ratesettingsmode_data.SetStringSelection(self.configfileargs['gui_ratesettingsmode'])

        if (sys.platform == 'win32'):
            win32_taskbar_icon_checkbox = wxCheckBox(panel, -1, "Minimize to system tray")
            win32_taskbar_icon_checkbox.SetValue(self.configfileargs['win32_taskbar_icon'])

        self.checkingcolor_icon = wxEmptyBitmap(16,16)
        self.checkingcolor_iconptr = wxStaticBitmap(panel, -1, self.checkingcolor_icon)
        self.setColorIcon(self.checkingcolor_icon, self.checkingcolor_iconptr, self.checkingcolormenu)

        self.downloadcolor_icon = wxEmptyBitmap(16,16)
        self.downloadcolor_iconptr = wxStaticBitmap(panel, -1, self.downloadcolor_icon)
        self.setColorIcon(self.downloadcolor_icon, self.downloadcolor_iconptr, self.downloadcolormenu)

        self.seedingcolor_icon = wxEmptyBitmap(16,16)
        self.seedingcolor_iconptr = wxStaticBitmap(panel, -1, self.seedingcolor_icon)
        self.setColorIcon(self.seedingcolor_icon, self.downloadcolor_iconptr, self.seedingcolormenu)
        
        rowsizer = wxFlexGridSizer(cols = 2, hgap = 20)

        block12sizer = wxFlexGridSizer(cols = 1)
        block12combosizer = wxFlexGridSizer(cols = 2, hgap = 20)

        block1sizer = wxFlexGridSizer(cols = 1, vgap = 2)
#        block1sizer.Add(wxStaticText(panel, -1, ' '))
        if (sys.platform == 'win32'):
            block1sizer.Add(win32_taskbar_icon_checkbox)
        block1sizer.Add(gui_stretchwindow_checkbox)
        block1sizer.Add(gui_displaystats_checkbox)
        block1sizer.Add(gui_displaymiscstats_checkbox)
        block1sizer.Add(gui_forcegreenonfirewall_checkbox)
        block1sizer.Add(wxStaticText(panel, -1, ' '))

        block12combosizer.Add(block1sizer)

        portsettingsSizer = wxStaticBoxSizer(wxStaticBox(panel, -1, "Port Range:*"), wxVERTICAL)
        portsettingsSizer1 = wxGridSizer(cols = 2, vgap = 1)
        portsettingsSizer1.Add(wxStaticText(panel, -1, 'From: '), 1, wxALIGN_CENTER_VERTICAL|wxALIGN_RIGHT)
        portsettingsSizer1.Add(minport_data, 1, wxALIGN_BOTTOM)
        portsettingsSizer1.Add(wxStaticText(panel, -1, 'To: '), 1, wxALIGN_CENTER_VERTICAL|wxALIGN_RIGHT)
        portsettingsSizer1.Add(maxport_data, 1, wxALIGN_BOTTOM)
#        minsize = minport_data.GetBestSize()
#        minsize.SetHeight(2*minsize.GetHeight()+5)
#        portsettingsSizer1.SetMinSize(minsize)
        portsettingsSizer.Add(portsettingsSizer1)

        block12combosizer.Add(portsettingsSizer)
        block12sizer.Add(block12combosizer)

        colorsizer = wxStaticBoxSizer(wxStaticBox(panel, -1, "Gauge Colors:"), wxVERTICAL)
        colorsizer1 = wxFlexGridSizer(cols = 7)
        colorsizer1.Add(wxStaticText(panel, -1, '           Checking: '), 1, wxALIGN_BOTTOM)
        colorsizer1.Add(self.checkingcolor_iconptr, 1, wxALIGN_BOTTOM)
        colorsizer1.Add(wxStaticText(panel, -1, '   Downloading: '), 1, wxALIGN_BOTTOM)
        colorsizer1.Add(self.downloadcolor_iconptr, 1, wxALIGN_BOTTOM)
        colorsizer1.Add(wxStaticText(panel, -1, '   Seeding: '), 1, wxALIGN_BOTTOM)
        colorsizer1.Add(self.seedingcolor_iconptr, 1, wxALIGN_BOTTOM)
        colorsizer1.Add(wxStaticText(panel, -1, '  '))
        minsize = self.checkingcolor_iconptr.GetBestSize()
        minsize.SetHeight(minsize.GetHeight()+5)
        colorsizer1.SetMinSize(minsize)

        colorsizer.Add(colorsizer1)
       
        block12sizer.Add(colorsizer, 1, wxALIGN_LEFT)
        rowsizer.Add(block12sizer)

        block3sizer = wxFlexGridSizer(cols = 1)
        block3sizer.Add(gui_ratesettingsmode_data, 1, wxALIGN_CENTER)
        ratesettingsSizer = wxGridSizer(cols = 1, vgap=1)
        ratesettingsSizer.Add(wxStaticText(panel, -1, 'Default Rate Setting: *'), 1, wxALIGN_CENTER)
        ratesettingsSizer.Add(gui_ratesettingsdefault_data, 1, wxALIGN_CENTER)
        block3sizer.Add(ratesettingsSizer, 1, wxALIGN_CENTER)
        
        rowsizer.Add(block3sizer)
        colsizer.Add(wxStaticText(panel, -1, ' '))
        colsizer.Add(rowsizer)


        savesizer = wxGridSizer(cols = 3, hgap = 20)
        saveButton = wxButton(panel, -1, 'Save')
        savesizer.Add(saveButton, 0, wxALIGN_CENTER)

        cancelButton = wxButton(panel, -1, 'Cancel')
        savesizer.Add(cancelButton, 0, wxALIGN_CENTER)

        defaultsButton = wxButton(panel, -1, 'Revert to Defaults')
        savesizer.Add(defaultsButton, 0, wxALIGN_CENTER)
        colsizer.Add(savesizer, 1, wxALIGN_CENTER)

        resizewarningtext=wxStaticText(panel, -1, '* These settings will not take effect until the next time you start BitTorrent')
        resizewarningtext.SetFont(wxFont(8, wxDEFAULT, wxNORMAL, wxNORMAL, false))
        colsizer.Add(resizewarningtext, 1, wxALIGN_CENTER)

        border = wxBoxSizer(wxHORIZONTAL)
        border.Add(colsizer, 1, wxEXPAND | wxALL, 4)
        
        panel.SetSizer(border)
        panel.SetAutoLayout(true)

        ref = self


        def setDefaults(self, frame = self):
            minport_data.SetValue(ref.defaults.get('minport'))
            maxport_data.SetValue(ref.defaults.get('maxport'))
            gui_stretchwindow_checkbox.SetValue(ref.configFileDefaults['gui_stretchwindow'])
            gui_displaystats_checkbox.SetValue(ref.configFileDefaults['gui_displaystats'])
            gui_displaymiscstats_checkbox.SetValue(ref.configFileDefaults['gui_displaymiscstats'])
            gui_forcegreenonfirewall_checkbox.SetValue(ref.configFileDefaults['gui_forcegreenonfirewall'])
            gui_ratesettingsdefault_data.SetStringSelection(ref.configFileDefaults['gui_ratesettingsdefault'])
            gui_ratesettingsmode_data.SetStringSelection(ref.configFileDefaults['gui_ratesettingsmode'])

            ref.checkingcolormenu = ref.HexToColor(ref.configFileDefaults['gui_checkingcolor'])
            ref.setColorIcon(ref.checkingcolor_icon, ref.checkingcolor_iconptr, ref.checkingcolormenu)
            ref.downloadcolormenu = ref.HexToColor(ref.configFileDefaults['gui_downloadcolor'])
            ref.setColorIcon(ref.downloadcolor_icon, ref.downloadcolor_iconptr, ref.downloadcolormenu)
            ref.seedingcolormenu = ref.HexToColor(ref.configFileDefaults['gui_seedingcolor'])
            ref.setColorIcon(ref.seedingcolor_icon, ref.seedingcolor_iconptr, ref.seedingcolormenu)

            if (sys.platform == 'win32'):
                win32_taskbar_icon_checkbox.SetValue(ref.configFileDefaults['win32_taskbar_icon'])


        def saveConfigs(self, frame = self):
            ref.configfileargs['gui_stretchwindow']=int(gui_stretchwindow_checkbox.GetValue())
            ref.configfileargs['gui_displaystats']=int(gui_displaystats_checkbox.GetValue())
            ref.configfileargs['gui_displaymiscstats']=int(gui_displaymiscstats_checkbox.GetValue())
            ref.configfileargs['gui_forcegreenonfirewall']=int(gui_forcegreenonfirewall_checkbox.GetValue())
            ref.configfileargs['minport']=minport_data.GetValue()
            ref.configfileargs['maxport']=maxport_data.GetValue()
            ref.configfileargs['gui_ratesettingsdefault']=gui_ratesettingsdefault_data.GetStringSelection()
            ref.configfileargs['gui_ratesettingsmode']=gui_ratesettingsmode_data.GetStringSelection()

            ref.checkingcolor = ref.checkingcolormenu
            ref.configfileargs['gui_checkingcolor']=ref.ColorToHex(ref.checkingcolor)
            ref.downloadcolor = ref.downloadcolormenu
            ref.configfileargs['gui_downloadcolor']=ref.ColorToHex(ref.downloadcolor)
            ref.seedingcolor = ref.seedingcolormenu
            ref.configfileargs['gui_seedingcolor']=ref.ColorToHex(ref.seedingcolor)
            
            if (sys.platform == 'win32'):
                ref.configfileargs['win32_taskbar_icon']=int(win32_taskbar_icon_checkbox.GetValue())
            ref.writeConfigFile()
            ref.configReset = true
            frame.configMenuBox.Close ()

        def cancelConfigs(self, frame = self):            
            frame.configMenuBox.Close ()

        def checkingcoloricon_set(self, frame = self):
            newcolor = ref.getColorFromUser(ref.panel,ref.checkingcolormenu)
            ref.setColorIcon(ref.checkingcolor_icon, ref.checkingcolor_iconptr, newcolor)
            ref.checkingcolormenu = newcolor

        def downloadcoloricon_set(self, frame = self):
            newcolor = ref.getColorFromUser(ref.panel,ref.downloadcolormenu)
            ref.setColorIcon(ref.downloadcolor_icon, ref.downloadcolor_iconptr, newcolor)
            ref.downloadcolormenu = newcolor

        def seedingcoloricon_set(self, frame = self):
            newcolor = ref.getColorFromUser(ref.panel,ref.seedingcolormenu)
            ref.setColorIcon(ref.seedingcolor_icon, ref.seedingcolor_iconptr, newcolor)
            ref.seedingcolormenu = newcolor


            
        EVT_BUTTON(self.configMenuBox, saveButton.GetId(), saveConfigs)
        EVT_BUTTON(self.configMenuBox, cancelButton.GetId(), cancelConfigs)
        EVT_BUTTON(self.configMenuBox, defaultsButton.GetId(), setDefaults)
        EVT_LEFT_DOWN(self.checkingcolor_iconptr, checkingcoloricon_set)
        EVT_LEFT_DOWN(self.downloadcolor_iconptr, downloadcoloricon_set)
        EVT_LEFT_DOWN(self.seedingcolor_iconptr, seedingcoloricon_set)


        self.configMenuBox.Show ()
        border.Fit(panel)
        self.configMenuBox.Fit()


    def Close(self):
        if (self.configMenuBox is not None):
            try:
                self.configMenuBox.Close ()
            except wxPyDeadObjectError, e:
                self.configMenuBox = None
